%% This file is used to calibrate the model in Rubio and Yao 
%  
clear all;

global M_ oo_

modnam = 'ico';
modnamstar = 'ico_zlb';

constraint = 'r<1 - RSS';
constraint_relax ='r>1 - RSS';

% Pick innovation for IRFs
irfshock =char('eps_d', 'eps_u');      % label for demand shock IRFs'eps_d',, 'eps_e'

maxiter = 50;
tol0 = 1e-8;

nperiods = 1000;

% set parameter values
rho_d = .22; % persistence of demand shocks
TECH =.95;  % persistence of productivity shocks


BETA = 0.998;   % SS interest rate
BETATILDE = 0.988;

phib = 0;    % macro-pru rule parameter
phiy = 0;

save parameterfile.mat phib phiy BETA BETATILDE TECH rho_d;

% do simulation

nsim = 1;

variance_simu = zeros(nsim, 7);
crosscorr_simu = zeros(nsim, 7);
autocorr_simu = zeros(nsim, 7);
duration_zlb = zeros(nsim, 1);
freq_zlb = zeros(nsim, 1);
prob_zlb = zeros(nsim, 1);

for i=1:nsim
    
   fprintf("progress: %d of %d \n", i, nsim);

% change seed for randn() each time it runs
   RandStream.setGlobalStream(RandStream('mt19937ar','seed',sum(100*clock)));
   SHOCKS  = zeros(nperiods,2);
   SHOCKS(:,1) = randn(nperiods,1) * 0.0023;  % demand shock
   SHOCKS(:,2) = randn(nperiods,1) * 0.0045;  % productivity shock
   %SHOCKS(:,3) = randn(nperiods,1) * 0.0025;  % monetary policy shock
   shockssequence = SHOCKS;

% Solve model, generate model IRFs
  [zdatalinear zdatapiecewise zdatass violvecbool_ oobase_ Mbase_  ] = ...
  solve_one_constraint(modnam,modnamstar,...
  constraint, constraint_relax,...
  shockssequence,irfshock,nperiods);

% unpack the IRFs                          
    for j=1:M_.endo_nbr
        eval([deblank(M_.endo_names(j,:)),'_p=zdatapiecewise(:,j);']);
    end
       
% save results1 y_p dp_p q_p r_p bc_p kt_p

% Report Business Cycle Statistics

   simudata = [y_p c_p x_p dp_p*4 q_p bc_p r_p*4]*100; % Consumption, GDP,	Resid. investment, House price, HH credit, FFR, Inflation
   
% simulated moments

   variance_simu(i, :) = var(simudata);
   cc = corr(simudata);
   crosscorr_simu(i, :) = cc(1,:);
   
   [acf,~,~] = autocorr(c_p, 1);
   autocorr_simu(i, 1) = acf(2);   
   [acf,~,~] = autocorr(y_p, 1);
   autocorr_simu(i, 2) = acf(2);
   [acf,~,~] = autocorr(x_p, 1);
   autocorr_simu(i, 3) = acf(2);
   [acf,~,~] = autocorr(q_p, 1);
   autocorr_simu(i, 4) = acf(2);
   [acf,~,~] = autocorr(bc_p, 1);
   autocorr_simu(i, 5) = acf(2);
   [acf,~,~] = autocorr(r_p, 1);
   autocorr_simu(i, 6) = acf(2);
   [acf,~,~] = autocorr(dp_p, 1);
   autocorr_simu(i, 7) = acf(2);
   
   
% unconditional probability of binding regime
    regime = zeros(size(r_p,1),1);
    for ii = 1: size(r_p,1)
        regime(ii) = (r_p(ii)+(RSS-1)<= 0);
    end

% average duration of binding regime
    count = 0; % number of binding regime
    for jj = 1: size(r_p,1)-1
        if regime(jj)==1 && regime(jj+1)==0
            count = count +1;
        end
    end

   duration_zlb(i) = sum(regime)/count;
   freq_zlb(i) = count;
   prob_zlb(i) = sum(regime)/size(r_p,1);

end



%save simu_995 variance_simu crosscorr_simu autocorr_simu duration_zlb freq_zlb prob_zlb ;

% 
% load simu_997;
% 
% simudata = [c_p y_p x_p q_p bc_p r_p dp_p];
% mean(variance_simu(1:3321,7)*10000)
% mean(variance_simu(1:3321,7)*10000)/mean(variance_simu(1:3321,2)*100)
% var(variance_simu(1:3321,7)*10000)


% 
% 
% %% Calculating the relevant stattistics
% 
% %Standard Deviations relative to GDPt: mean and st dev. across all simulations
% % SimRER=[mean((std(drer')./std(dGDP')))];
% % SimC=[mean((std(dC')./std(dGDP')))];
% SimInv=[mean((std(dI')./std(dY')))];
% 
% % outRER = SimRER-targets(1);
% %outC = SimC-targets(2);
% outI = SimInv-targets(1);
% 


% %% Plot
% st = 1;    % start period of plotting
% ed = 1000; % end period of plotting
% TT = size(r_p(st:ed),1);
% figure()
% 
% subplot(2,2,1)
% plot((r_p(st:ed)+(RSS-1))*400,'k','LineWidth', 1.)
% hold on
% %plot((r_p_4(st:ed)+(1.0101-1))*400,'--','LineWidth', 1.)
% plot( zeros(TT+1,1),'-k','LineWidth', 1);
% title('Policy rate')
% ylabel('Annualized Level, PPt')
%  xlim([0,TT])
%  %legend('2% S.S. interest rate','4% S.S. interest rate)')
%  
% subplot(2,2,2)
% plot( 100*y_p(st:ed),'k','LineWidth', 1.)
%  hold on
%  %plot( 100*y_p_4(st:ed),'--','LineWidth', 1.)
%  plot( zeros(TT+1,1),'-k','LineWidth', 1);
%  title('\bfOutput','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,TT])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  
%  
% subplot(2,2,3)
%  plot( 100*q_p(st:ed),'k','LineWidth', 1.)
%  hold on
%  %plot( 100*q_p_4(st:ed),'--','LineWidth', 1.)
%  plot( zeros(TT+1,1),'-k','LineWidth', 1);
%  title('\bfHouse Price','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,TT]) 
%  
% subplot(2,2,4)
% plot( 100*bc_p(st:ed),'k','LineWidth', 1.)
%  hold on
% % plot( 100*bc_p_4(st:ed),'--','LineWidth', 1.)
%  plot( zeros(TT+1,1),'-k','LineWidth', 1);
%  title('\bfDebt','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,TT])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  %legend('Occbin','no ZLB')
%  