function [residual, g1, g2, g3] = ico_zlb_dynamic(y, x, params, steady_state, it_)
%
% Status : Computes dynamic model for Dynare
%
% Inputs :
%   y         [#dynamic variables by 1] double    vector of endogenous variables in the order stored
%                                                 in M_.lead_lag_incidence; see the Manual
%   x         [nperiods by M_.exo_nbr] double     matrix of exogenous variables (in declaration order)
%                                                 for all simulation periods
%   steady_state  [M_.endo_nbr by 1] double       vector of steady state values
%   params    [M_.param_nbr by 1] double          vector of parameter values in declaration order
%   it_       scalar double                       time period for exogenous variables for which to evaluate the model
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the dynamic model equations in order of 
%                                          declaration of the equations.
%                                          Dynare may prepend auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by #dynamic variables] double    Jacobian matrix of the dynamic model equations;
%                                                           rows: equations in order of declaration
%                                                           columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g2        [M_.endo_nbr by (#dynamic variables)^2] double   Hessian matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g3        [M_.endo_nbr by (#dynamic variables)^3] double   Third order derivative matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

%
% Model equations
%

residual = zeros(20, 1);
T59 = (params(17)+(1-params(2))*params(3)*params(5))/params(17);
T88 = params(2)*params(5)/params(17);
T98 = params(1)/params(17)+params(5)/params(17)*(params(2)-params(1));
lhs =(-y(11));
rhs =y(21)-y(32)-y(29)+y(28);
residual(1)= lhs-rhs;
lhs =y(18);
rhs =y(11)+(params(4)-1)*y(16);
residual(2)= lhs-rhs;
lhs =y(19);
rhs =(params(4)-1)*y(17)+y(12);
residual(3)= lhs-rhs;
lhs =y(14);
rhs =1/(1-params(2))*(y(11)-y(24))-params(2)/(1-params(2))*(y(29)-y(33));
residual(4)= lhs-rhs;
lhs =y(20);
rhs =y(32)+y(33)+y(27)+y(15)-y(31);
residual(5)= lhs-rhs;
lhs =y(12);
rhs =T59*(y(9)-y(10))-params(3)/params(17)*(y(15)-y(2))+params(3)*params(5)/params(17)*(params(2)*y(20)-y(3))-params(3)*params(5)*(y(4)-y(23));
residual(6)= lhs-rhs;
lhs =y(15);
rhs =y(12)*(1-params(2)*params(5))/params(17)-y(24)*1/params(17)+T88*(y(32)+y(27)-y(21))+y(33)*T98-params(1)*(1-params(5))/params(17)*y(30)-y(27)*params(5)*params(1)/params(17);
residual(7)= lhs-rhs;
lhs =y(13);
rhs =y(12)*params(19)+y(11)*params(20);
residual(8)= lhs-rhs;
lhs =0;
rhs =y(14)*params(22)+y(15)*params(21);
residual(9)= lhs-rhs;
lhs =y(9);
rhs =y(25)+y(16)*params(6)+y(17)*(1-params(6));
residual(10)= lhs-rhs;
lhs =y(18);
rhs =y(9)-y(10)-y(16);
residual(11)= lhs-rhs;
lhs =y(19);
rhs =y(9)-y(10)-y(17);
residual(12)= lhs-rhs;
lhs =y(23);
rhs =y(32)*params(2)-y(10)*params(9);
residual(13)= lhs-rhs;
lhs =y(22);
rhs =params(10)*y(5)+y(23)*(1-params(10))*(1+params(11))+(1-params(10))*params(12)*(y(9)-y(1))+y(26);
residual(14)= lhs-rhs;
lhs =y(21);
rhs =1-params(23);
residual(15)= lhs-rhs;
lhs =y(9);
rhs =y(13);
residual(16)= lhs-rhs;
lhs =y(25);
rhs =params(13)*y(6)+x(it_, 1);
residual(17)= lhs-rhs;
lhs =y(26);
rhs =0.0*y(7)+x(it_, 2);
residual(18)= lhs-rhs;
lhs =y(27);
rhs =y(20)*(-params(14))-y(9)*params(15);
residual(19)= lhs-rhs;
lhs =y(28);
rhs =params(24)*y(8)+x(it_, 3);
residual(20)= lhs-rhs;
if nargout >= 2,
  g1 = zeros(20, 36);

  %
  % Jacobian matrix
  %

  g1(1,11)=(-1);
  g1(1,29)=1;
  g1(1,21)=(-1);
  g1(1,32)=1;
  g1(1,28)=(-1);
  g1(2,11)=(-1);
  g1(2,16)=(-(params(4)-1));
  g1(2,18)=1;
  g1(3,12)=(-1);
  g1(3,17)=(-(params(4)-1));
  g1(3,19)=1;
  g1(4,11)=(-(1/(1-params(2))));
  g1(4,29)=params(2)/(1-params(2));
  g1(4,14)=1;
  g1(4,24)=1/(1-params(2));
  g1(4,33)=(-(params(2)/(1-params(2))));
  g1(5,15)=(-1);
  g1(5,20)=1;
  g1(5,31)=1;
  g1(5,32)=(-1);
  g1(5,33)=(-1);
  g1(5,27)=(-1);
  g1(6,9)=(-T59);
  g1(6,10)=T59;
  g1(6,12)=1;
  g1(6,2)=(-(params(3)/params(17)));
  g1(6,15)=params(3)/params(17);
  g1(6,3)=params(3)*params(5)/params(17);
  g1(6,20)=(-(params(2)*params(3)*params(5)/params(17)));
  g1(6,4)=params(3)*params(5);
  g1(6,23)=(-(params(3)*params(5)));
  g1(7,12)=(-((1-params(2)*params(5))/params(17)));
  g1(7,30)=params(1)*(1-params(5))/params(17);
  g1(7,15)=1;
  g1(7,21)=T88;
  g1(7,32)=(-T88);
  g1(7,24)=1/params(17);
  g1(7,33)=(-T98);
  g1(7,27)=(-(T88-params(5)*params(1)/params(17)));
  g1(8,11)=(-params(20));
  g1(8,12)=(-params(19));
  g1(8,13)=1;
  g1(9,14)=(-params(22));
  g1(9,15)=(-params(21));
  g1(10,9)=1;
  g1(10,16)=(-params(6));
  g1(10,17)=(-(1-params(6)));
  g1(10,25)=(-1);
  g1(11,9)=(-1);
  g1(11,10)=1;
  g1(11,16)=1;
  g1(11,18)=1;
  g1(12,9)=(-1);
  g1(12,10)=1;
  g1(12,17)=1;
  g1(12,19)=1;
  g1(13,10)=params(9);
  g1(13,23)=1;
  g1(13,32)=(-params(2));
  g1(14,1)=(1-params(10))*params(12);
  g1(14,9)=(-((1-params(10))*params(12)));
  g1(14,5)=(-params(10));
  g1(14,22)=1;
  g1(14,23)=(-((1-params(10))*(1+params(11))));
  g1(14,26)=(-1);
  g1(15,21)=1;
  g1(16,9)=1;
  g1(16,13)=(-1);
  g1(17,6)=(-params(13));
  g1(17,25)=1;
  g1(17,34)=(-1);
  g1(18,7)=(-0.0);
  g1(18,26)=1;
  g1(18,35)=(-1);
  g1(19,9)=params(15);
  g1(19,20)=params(14);
  g1(19,27)=1;
  g1(20,8)=(-params(24));
  g1(20,28)=1;
  g1(20,36)=(-1);

if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],20,1296);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],20,46656);
end
end
end
end
