function [residual, g1, g2, g3] = ico_zlb_static(y, x, params)
%
% Status : Computes static model for Dynare
%
% Inputs : 
%   y         [M_.endo_nbr by 1] double    vector of endogenous variables in declaration order
%   x         [M_.exo_nbr by 1] double     vector of exogenous variables in declaration order
%   params    [M_.param_nbr by 1] double   vector of parameter values in declaration order
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the static model equations 
%                                          in order of declaration of the equations.
%                                          Dynare may prepend or append auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by M_.endo_nbr] double    Jacobian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g2        [M_.endo_nbr by (M_.endo_nbr)^2] double   Hessian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g3        [M_.endo_nbr by (M_.endo_nbr)^3] double   Third derivatives matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

residual = zeros( 20, 1);

%
% Model equations
%

T55 = (params(17)+(1-params(2))*params(3)*params(5))/params(17);
T76 = params(2)*params(5)/params(17);
T86 = params(1)/params(17)+params(5)/params(17)*(params(2)-params(1));
lhs =(-y(3));
rhs =y(13)-y(15)-y(3)+y(20);
residual(1)= lhs-rhs;
lhs =y(10);
rhs =y(3)+(params(4)-1)*y(8);
residual(2)= lhs-rhs;
lhs =y(11);
rhs =(params(4)-1)*y(9)+y(4);
residual(3)= lhs-rhs;
lhs =y(6);
rhs =1/(1-params(2))*(y(3)-y(16))-(y(3)-y(16))*params(2)/(1-params(2));
residual(4)= lhs-rhs;
lhs =y(12);
rhs =y(15)+y(16)+y(19)+y(7)-y(13);
residual(5)= lhs-rhs;
lhs =y(4);
rhs =T55*(y(1)-y(2))+params(3)*params(5)/params(17)*(params(2)*y(12)-y(12))-(y(13)-y(15))*params(3)*params(5);
residual(6)= lhs-rhs;
lhs =y(7);
rhs =y(4)*(1-params(2)*params(5))/params(17)-y(16)*1/params(17)+T76*(y(15)+y(19)-y(13))+y(16)*T86-y(4)*params(1)*(1-params(5))/params(17)-y(19)*params(5)*params(1)/params(17);
residual(7)= lhs-rhs;
lhs =y(5);
rhs =y(4)*params(19)+y(3)*params(20);
residual(8)= lhs-rhs;
lhs =0;
rhs =y(6)*params(22)+y(7)*params(21);
residual(9)= lhs-rhs;
lhs =y(1);
rhs =y(17)+y(8)*params(6)+y(9)*(1-params(6));
residual(10)= lhs-rhs;
lhs =y(10);
rhs =y(1)-y(2)-y(8);
residual(11)= lhs-rhs;
lhs =y(11);
rhs =y(1)-y(2)-y(9);
residual(12)= lhs-rhs;
lhs =y(15);
rhs =y(15)*params(2)-y(2)*params(9);
residual(13)= lhs-rhs;
lhs =y(14);
rhs =y(14)*params(10)+y(15)*(1-params(10))*(1+params(11))+y(18);
residual(14)= lhs-rhs;
lhs =y(13);
rhs =1-params(23);
residual(15)= lhs-rhs;
lhs =y(1);
rhs =y(5);
residual(16)= lhs-rhs;
lhs =y(17);
rhs =y(17)*params(13)+x(1);
residual(17)= lhs-rhs;
lhs =y(18);
rhs =y(18)*0.0+x(2);
residual(18)= lhs-rhs;
lhs =y(19);
rhs =y(12)*(-params(14))-y(1)*params(15);
residual(19)= lhs-rhs;
lhs =y(20);
rhs =y(20)*params(24)+x(3);
residual(20)= lhs-rhs;
if ~isreal(residual)
  residual = real(residual)+imag(residual).^2;
end
if nargout >= 2,
  g1 = zeros(20, 20);

  %
  % Jacobian matrix
  %

  g1(1,13)=(-1);
  g1(1,15)=1;
  g1(1,20)=(-1);
  g1(2,3)=(-1);
  g1(2,8)=(-(params(4)-1));
  g1(2,10)=1;
  g1(3,4)=(-1);
  g1(3,9)=(-(params(4)-1));
  g1(3,11)=1;
  g1(4,3)=(-(1/(1-params(2))-params(2)/(1-params(2))));
  g1(4,6)=1;
  g1(4,16)=(-((-(1/(1-params(2))))-(-(params(2)/(1-params(2))))));
  g1(5,7)=(-1);
  g1(5,12)=1;
  g1(5,13)=1;
  g1(5,15)=(-1);
  g1(5,16)=(-1);
  g1(5,19)=(-1);
  g1(6,1)=(-T55);
  g1(6,2)=T55;
  g1(6,4)=1;
  g1(6,12)=(-(params(3)*params(5)/params(17)*(params(2)-1)));
  g1(6,13)=params(3)*params(5);
  g1(6,15)=(-(params(3)*params(5)));
  g1(7,4)=(-((1-params(2)*params(5))/params(17)-params(1)*(1-params(5))/params(17)));
  g1(7,7)=1;
  g1(7,13)=T76;
  g1(7,15)=(-T76);
  g1(7,16)=(-(T86+(-(1/params(17)))));
  g1(7,19)=(-(T76-params(5)*params(1)/params(17)));
  g1(8,3)=(-params(20));
  g1(8,4)=(-params(19));
  g1(8,5)=1;
  g1(9,6)=(-params(22));
  g1(9,7)=(-params(21));
  g1(10,1)=1;
  g1(10,8)=(-params(6));
  g1(10,9)=(-(1-params(6)));
  g1(10,17)=(-1);
  g1(11,1)=(-1);
  g1(11,2)=1;
  g1(11,8)=1;
  g1(11,10)=1;
  g1(12,1)=(-1);
  g1(12,2)=1;
  g1(12,9)=1;
  g1(12,11)=1;
  g1(13,2)=params(9);
  g1(13,15)=1-params(2);
  g1(14,14)=1-params(10);
  g1(14,15)=(-((1-params(10))*(1+params(11))));
  g1(14,18)=(-1);
  g1(15,13)=1;
  g1(16,1)=1;
  g1(16,5)=(-1);
  g1(17,17)=1-params(13);
  g1(18,18)=1;
  g1(19,1)=params(15);
  g1(19,12)=params(14);
  g1(19,19)=1;
  g1(20,20)=1-params(24);
  if ~isreal(g1)
    g1 = real(g1)+2*imag(g1);
  end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],20,400);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],20,8000);
end
end
end
end
