%% This file is used to calibrate the model in Rubio and Yao 
%  
clear

global M_ oo_

modnam = 'ico';
modnamstar = 'ico_zlb';

constraint = 'r<1 - RSS';
constraint_relax ='r>1 - RSS';

% Pick innovation for IRFs
irfshock =char('eps_d', 'eps_u');      % label for demand shock IRFs'eps_d',

maxiter = 50;
tol0 = 1e-8;

nperiods = 1000;

% set parameter values

TECH =.95;  % persistence of productivity shocks
rho_d = .22; % persistence of demand shocks

BETA = 0.997;   % SS interest rate
BETATILDE = 0.987;

phib = 0;    % macro-
phiy = 0;
save parameterfile.mat phib phiy BETA BETATILDE TECH rho_d;

% do simulation
% change seed for randn() each time it runs
RandStream.setGlobalStream(RandStream('mt19937ar','seed',sum(100*clock)));
SHOCKS  = zeros(nperiods,2);
SHOCKS(:,1) = randn(nperiods,1) * 0.0023;  % demand shock
SHOCKS(:,2) = randn(nperiods,1) * 0.0045;  % productivity shock
shockssequence = SHOCKS;

% Solve model, generate model IRFs
[zdatalinear zdatapiecewise zdatass violvecbool_ oobase_ Mbase_  ] = ...
  solve_one_constraint(modnam,modnamstar,...
  constraint, constraint_relax,...
  shockssequence,irfshock,nperiods);


% unpack the IRFs                          
for i=1:M_.endo_nbr
  eval([deblank(M_.endo_names(i,:)),'_u=zdatalinear(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_ss=zdatass(i);']);
end

          
%save results1 y_p dp_p q_p r_p bc_p kt_p

% Report Business Cycle Statistics

%axa = [dy_obs,dc_obs,dinv_obs,dx_obs,ffr_obs,pi_obs,dhp_obs,hrs_obs,spread_obs,dy_sim];

simudata = [y_p dp_p q_p r_p bc_p kt_p];

% simulated moments

%mean(simudata)*100
disp('Variance');
var(simudata)*100

disp('Cross-correlation');
corr(simudata)*100

disp('Auto-correlation');
[acf,lags,bounds] = autocorr(y_p, 5);
acf

% unconditional probability of binding regime
regime = zeros(size(r_p,1),1);
for i = 1: size(r_p,1)
    regime(i) = (r_p(i)+(RSS-1)<= 0);
end

prob_binding = sum(regime)/size(r_p,1);

% average duration of binding regime
count = 0; % number of binding regime
for i = 1: size(r_p,1)-1
    if regime(i)==1 && regime(i+1)==0
        count = count +1;
    end
end

duration =  sum(regime)/count;

fprintf("average duration: %.2f\n", duration);
fprintf("number of binding regime: %d\n", count);
fprintf("Probability of binding regime: %.4f \n", prob_binding);

% Plot
st = 1;    % start period of plotting
ed = 1000; % end period of plotting
TT = size(r_p(st:ed),1);
figure()

subplot(2,2,1)
plot((r_p(st:ed)+(RSS-1))*400,'k','LineWidth', 1.)
hold on
%plot((r_p_4(st:ed)+(1.0101-1))*400,'--','LineWidth', 1.)
plot( zeros(TT+1,1),'-k','LineWidth', 1);
title('Policy rate')
ylabel('Annualized Level, PPt')
 xlim([0,TT])
 %legend('2% S.S. interest rate','4% S.S. interest rate)')
 
subplot(2,2,2)
plot( 100*y_p(st:ed),'k','LineWidth', 1.)
 hold on
 %plot( 100*y_p_4(st:ed),'--','LineWidth', 1.)
 plot( zeros(TT+1,1),'-k','LineWidth', 1);
 title('\bfOutput','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,TT])
 %ylim([0,3])
 %csPatches(gcf,bindingdates)
 
 
subplot(2,2,3)
 plot( 100*q_p(st:ed),'k','LineWidth', 1.)
 hold on
 %plot( 100*q_p_4(st:ed),'--','LineWidth', 1.)
 plot( zeros(TT+1,1),'-k','LineWidth', 1);
 title('\bfHouse Price','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([0,TT]) 
 
subplot(2,2,4)
plot( 100*bc_p(st:ed),'k','LineWidth', 1.)
 hold on
% plot( 100*bc_p_4(st:ed),'--','LineWidth', 1.)
 plot( zeros(TT+1,1),'-k','LineWidth', 1);
 title('\bfDebt','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,TT])
 %ylim([0,3])
 %csPatches(gcf,bindingdates)
 %legend('Occbin','no ZLB')
 

