%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_difference holds the piece-wise linear solution for
%  VariableName.  VariableName_uncdifference holds the linear solution for
%  VariableName.


clear


global M_ oo_

phib = 0;
phiy = 0;
save parameterfile.mat phib phiy;

modnam = 'ico';
modnamstar = 'ico_zlb';

constraint = 'r<1 - RSS';
constraint_relax ='r>1 - RSS';

% Pick innovation for IRFs
irfshock =char('eps_d');  % 'eps_u' tech shock;  'eps_d' for demand shock

maxiter = 200;
tol0 = 1e-8;

nperiods = 20;

% % For IRF
SHOCKS = zeros(nperiods,1);

%SHOCKS(2)= -randn(1,1) * 1;  % eps_u

SHOCKS(2)= randn(1,1) * .03;  % eps_d


shockssequence = SHOCKS;

% Solve model, generate model IRFs
[zdatalinear zdatapiecewise zdatass oobase_ Mbase_  ] = ...
  solve_one_constraint(modnam,modnamstar,...
  constraint, constraint_relax,...
  shockssequence,irfshock,nperiods);


% unpack the IRFs                          
for i=1:M_.endo_nbr
  eval([deblank(M_.endo_names(i,:)),'_u=zdatalinear(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_ss=zdatass(i);']);
end


%%
figure() % figure 1 for paper

subplot(3,2,1)
plot((r_p(1:end)+(RSS-1))*400,'r','LineWidth', 2)
hold on
plot((r_u(1:end)+(RSS-1))*400,'--b','LineWidth', 2)
plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
title('Policy rate')
ylabel('Annualized Level, PPt')
xlim([1,nperiods])
legend('ZLB (Occbin)','No ZLB')
  
  
subplot(3,2,2)
plot( 100*y_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*y_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfOutput','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %ylim([0,3])
 %csPatches(gcf,bindingdates)


subplot(3,2,3)
 plot(100*dp_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*dp_u(2:end),'--b','LineWidth', 2)
% plot( L_u,'-k','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfInflation','FontSize',12')
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %csPatches(gcf,bindingdates)
 
 subplot(3,2,4)
 plot( 100*rr_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*rr_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfReal interest rate','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([1,nperiods]) 
 
 subplot(3,2,5)
 plot( 100*q_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*q_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfHouse Price','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([1,nperiods]) 

 
 subplot(3,2,6)
 plot( 100*bc_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*bc_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfDebt','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 
