function [residual, g1, g2, g3] = ico_dynamic(y, x, params, steady_state, it_)
%
% Status : Computes dynamic model for Dynare
%
% Inputs :
%   y         [#dynamic variables by 1] double    vector of endogenous variables in the order stored
%                                                 in M_.lead_lag_incidence; see the Manual
%   x         [nperiods by M_.exo_nbr] double     matrix of exogenous variables (in declaration order)
%                                                 for all simulation periods
%   steady_state  [M_.endo_nbr by 1] double       vector of steady state values
%   params    [M_.param_nbr by 1] double          vector of parameter values in declaration order
%   it_       scalar double                       time period for exogenous variables for which to evaluate the model
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the dynamic model equations in order of 
%                                          declaration of the equations.
%                                          Dynare may prepend auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by #dynamic variables] double    Jacobian matrix of the dynamic model equations;
%                                                           rows: equations in order of declaration
%                                                           columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g2        [M_.endo_nbr by (#dynamic variables)^2] double   Hessian matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%   g3        [M_.endo_nbr by (#dynamic variables)^3] double   Third order derivative matrix of the dynamic model equations;
%                                                              rows: equations in order of declaration
%                                                              columns: variables in order stored in M_.lead_lag_incidence followed by the ones in M_.exo_names
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

%
% Model equations
%

residual = zeros(22, 1);
lhs =y(31)-y(11);
rhs =y(21)-y(33)+y(29);
residual(1)= lhs-rhs;
lhs =y(18);
rhs =y(11)+params(4)*y(16);
residual(2)= lhs-rhs;
lhs =y(19);
rhs =params(4)*y(17)+y(12);
residual(3)= lhs-rhs;
lhs =params(3)*params(17)*(y(11)-y(14));
rhs =params(26)*y(24)-params(26)*params(2)*(y(11)+y(34)-y(31));
residual(4)= lhs-rhs;
lhs =y(20);
rhs =y(33)+y(34)+y(27)+y(15)-y(21);
residual(5)= lhs-rhs;
lhs =y(12)+params(24)*params(18)*(y(4)+y(3)-y(23))+params(26)/params(17)*(y(15)-y(2));
rhs =y(20)*params(18)+(1-params(6))*(y(19)+y(17));
residual(6)= lhs-rhs;
lhs =params(3)*params(17)*(y(12)-y(15));
rhs =params(26)*y(24)-params(26)*params(1)*(y(34)+y(12)-y(32))-params(26)*(params(2)-params(1))*params(5)*(y(12)+y(33)+y(34)+y(27)+y(30));
residual(7)= lhs-rhs;
lhs =0;
rhs =params(2)*(y(12)+y(21)+y(30))-params(1)*(y(32)+y(33)+y(30));
residual(8)= lhs-rhs;
lhs =y(13);
rhs =y(11)*params(7)+y(12)*(1-params(7));
residual(9)= lhs-rhs;
lhs =0;
rhs =y(14)*params(7)+y(15)*(1-params(7));
residual(10)= lhs-rhs;
lhs =y(9);
rhs =y(25)+y(16)*params(6)+y(17)*(1-params(6));
residual(11)= lhs-rhs;
lhs =y(18);
rhs =y(9)-y(10)-y(16);
residual(12)= lhs-rhs;
lhs =y(19);
rhs =y(9)-y(10)-y(17);
residual(13)= lhs-rhs;
lhs =y(23);
rhs =y(33)*params(2)-y(10)*params(10);
residual(14)= lhs-rhs;
lhs =y(22);
rhs =params(11)*y(5)+y(23)*(1-params(11))*(1+params(12))+(1-params(11))*params(13)*(y(9)-y(1))+y(26);
residual(15)= lhs-rhs;
lhs =y(21);
rhs =y(22);
residual(16)= lhs-rhs;
lhs =y(28);
rhs =y(21)-y(33);
residual(17)= lhs-rhs;
lhs =y(9);
rhs =y(13);
residual(18)= lhs-rhs;
lhs =y(25);
rhs =params(14)*y(6)+x(it_, 1);
residual(19)= lhs-rhs;
lhs =y(26);
rhs =0.0*y(7)+x(it_, 2);
residual(20)= lhs-rhs;
lhs =y(27);
rhs =y(20)*(-params(15))-y(9)*params(16);
residual(21)= lhs-rhs;
lhs =y(29);
rhs =0.8*y(8)+x(it_, 3);
residual(22)= lhs-rhs;
if nargout >= 2,
  g1 = zeros(22, 37);

  %
  % Jacobian matrix
  %

  g1(1,11)=(-1);
  g1(1,31)=1;
  g1(1,21)=(-1);
  g1(1,33)=1;
  g1(1,29)=(-1);
  g1(2,11)=(-1);
  g1(2,16)=(-params(4));
  g1(2,18)=1;
  g1(3,12)=(-1);
  g1(3,17)=(-params(4));
  g1(3,19)=1;
  g1(4,11)=params(3)*params(17)-(-(params(26)*params(2)));
  g1(4,31)=(-(params(26)*params(2)));
  g1(4,14)=(-(params(3)*params(17)));
  g1(4,24)=(-params(26));
  g1(4,34)=params(26)*params(2);
  g1(5,15)=(-1);
  g1(5,20)=1;
  g1(5,21)=1;
  g1(5,33)=(-1);
  g1(5,34)=(-1);
  g1(5,27)=(-1);
  g1(6,12)=1;
  g1(6,2)=(-(params(26)/params(17)));
  g1(6,15)=params(26)/params(17);
  g1(6,17)=(-(1-params(6)));
  g1(6,19)=(-(1-params(6)));
  g1(6,3)=params(24)*params(18);
  g1(6,20)=(-params(18));
  g1(6,4)=params(24)*params(18);
  g1(6,23)=(-(params(24)*params(18)));
  g1(7,12)=params(3)*params(17)-((-(params(26)*params(1)))-params(26)*(params(2)-params(1))*params(5));
  g1(7,32)=(-(params(26)*params(1)));
  g1(7,15)=(-(params(3)*params(17)));
  g1(7,33)=params(26)*(params(2)-params(1))*params(5);
  g1(7,24)=(-params(26));
  g1(7,34)=(-((-(params(26)*params(1)))-params(26)*(params(2)-params(1))*params(5)));
  g1(7,27)=params(26)*(params(2)-params(1))*params(5);
  g1(7,30)=params(26)*(params(2)-params(1))*params(5);
  g1(8,12)=(-params(2));
  g1(8,32)=params(1);
  g1(8,21)=(-params(2));
  g1(8,33)=params(1);
  g1(8,30)=(-(params(2)-params(1)));
  g1(9,11)=(-params(7));
  g1(9,12)=(-(1-params(7)));
  g1(9,13)=1;
  g1(10,14)=(-params(7));
  g1(10,15)=(-(1-params(7)));
  g1(11,9)=1;
  g1(11,16)=(-params(6));
  g1(11,17)=(-(1-params(6)));
  g1(11,25)=(-1);
  g1(12,9)=(-1);
  g1(12,10)=1;
  g1(12,16)=1;
  g1(12,18)=1;
  g1(13,9)=(-1);
  g1(13,10)=1;
  g1(13,17)=1;
  g1(13,19)=1;
  g1(14,10)=params(10);
  g1(14,23)=1;
  g1(14,33)=(-params(2));
  g1(15,1)=(1-params(11))*params(13);
  g1(15,9)=(-((1-params(11))*params(13)));
  g1(15,5)=(-params(11));
  g1(15,22)=1;
  g1(15,23)=(-((1-params(11))*(1+params(12))));
  g1(15,26)=(-1);
  g1(16,21)=1;
  g1(16,22)=(-1);
  g1(17,21)=(-1);
  g1(17,33)=1;
  g1(17,28)=1;
  g1(18,9)=1;
  g1(18,13)=(-1);
  g1(19,6)=(-params(14));
  g1(19,25)=1;
  g1(19,35)=(-1);
  g1(20,7)=(-0.0);
  g1(20,26)=1;
  g1(20,36)=(-1);
  g1(21,9)=params(16);
  g1(21,20)=params(15);
  g1(21,27)=1;
  g1(22,8)=(-0.8);
  g1(22,29)=1;
  g1(22,37)=(-1);

if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],22,1369);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],22,50653);
end
end
end
end
