function [residual, g1, g2, g3] = ico_static(y, x, params)
%
% Status : Computes static model for Dynare
%
% Inputs : 
%   y         [M_.endo_nbr by 1] double    vector of endogenous variables in declaration order
%   x         [M_.exo_nbr by 1] double     vector of exogenous variables in declaration order
%   params    [M_.param_nbr by 1] double   vector of parameter values in declaration order
%
% Outputs:
%   residual  [M_.endo_nbr by 1] double    vector of residuals of the static model equations 
%                                          in order of declaration of the equations.
%                                          Dynare may prepend or append auxiliary equations, see M_.aux_vars
%   g1        [M_.endo_nbr by M_.endo_nbr] double    Jacobian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g2        [M_.endo_nbr by (M_.endo_nbr)^2] double   Hessian matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%   g3        [M_.endo_nbr by (M_.endo_nbr)^3] double   Third derivatives matrix of the static model equations;
%                                                       columns: variables in declaration order
%                                                       rows: equations in order of declaration
%
%
% Warning : this file is generated automatically by Dynare
%           from model file (.mod)

residual = zeros( 22, 1);

%
% Model equations
%

lhs =0;
rhs =y(13)-y(15)+y(21);
residual(1)= lhs-rhs;
lhs =y(10);
rhs =y(3)+params(4)*y(8);
residual(2)= lhs-rhs;
lhs =y(11);
rhs =params(4)*y(9)+y(4);
residual(3)= lhs-rhs;
lhs =params(3)*params(17)*(y(3)-y(6));
rhs =params(26)*y(16)-params(26)*params(2)*(y(3)+y(16)-y(3));
residual(4)= lhs-rhs;
lhs =y(12);
rhs =y(15)+y(16)+y(19)+y(7)-y(13);
residual(5)= lhs-rhs;
lhs =y(4)+params(24)*params(18)*(y(13)+y(12)-y(15));
rhs =y(12)*params(18)+(1-params(6))*(y(11)+y(9));
residual(6)= lhs-rhs;
lhs =params(3)*params(17)*(y(4)-y(7));
rhs =params(26)*y(16)-y(16)*params(26)*params(1)-params(26)*(params(2)-params(1))*params(5)*(y(4)+y(15)+y(16)+y(19)+y(22));
residual(7)= lhs-rhs;
lhs =0;
rhs =params(2)*(y(4)+y(13)+y(22))-params(1)*(y(4)+y(15)+y(22));
residual(8)= lhs-rhs;
lhs =y(5);
rhs =y(3)*params(7)+y(4)*(1-params(7));
residual(9)= lhs-rhs;
lhs =0;
rhs =y(6)*params(7)+y(7)*(1-params(7));
residual(10)= lhs-rhs;
lhs =y(1);
rhs =y(17)+y(8)*params(6)+y(9)*(1-params(6));
residual(11)= lhs-rhs;
lhs =y(10);
rhs =y(1)-y(2)-y(8);
residual(12)= lhs-rhs;
lhs =y(11);
rhs =y(1)-y(2)-y(9);
residual(13)= lhs-rhs;
lhs =y(15);
rhs =y(15)*params(2)-y(2)*params(10);
residual(14)= lhs-rhs;
lhs =y(14);
rhs =y(14)*params(11)+y(15)*(1-params(11))*(1+params(12))+y(18);
residual(15)= lhs-rhs;
lhs =y(13);
rhs =y(14);
residual(16)= lhs-rhs;
lhs =y(20);
rhs =y(13)-y(15);
residual(17)= lhs-rhs;
lhs =y(1);
rhs =y(5);
residual(18)= lhs-rhs;
lhs =y(17);
rhs =y(17)*params(14)+x(1);
residual(19)= lhs-rhs;
lhs =y(18);
rhs =y(18)*0.0+x(2);
residual(20)= lhs-rhs;
lhs =y(19);
rhs =y(12)*(-params(15))-y(1)*params(16);
residual(21)= lhs-rhs;
lhs =y(21);
rhs =y(21)*0.8+x(3);
residual(22)= lhs-rhs;
if ~isreal(residual)
  residual = real(residual)+imag(residual).^2;
end
if nargout >= 2,
  g1 = zeros(22, 22);

  %
  % Jacobian matrix
  %

  g1(1,13)=(-1);
  g1(1,15)=1;
  g1(1,21)=(-1);
  g1(2,3)=(-1);
  g1(2,8)=(-params(4));
  g1(2,10)=1;
  g1(3,4)=(-1);
  g1(3,9)=(-params(4));
  g1(3,11)=1;
  g1(4,3)=params(3)*params(17);
  g1(4,6)=(-(params(3)*params(17)));
  g1(4,16)=(-(params(26)-params(26)*params(2)));
  g1(5,7)=(-1);
  g1(5,12)=1;
  g1(5,13)=1;
  g1(5,15)=(-1);
  g1(5,16)=(-1);
  g1(5,19)=(-1);
  g1(6,4)=1;
  g1(6,9)=(-(1-params(6)));
  g1(6,11)=(-(1-params(6)));
  g1(6,12)=params(24)*params(18)-params(18);
  g1(6,13)=params(24)*params(18);
  g1(6,15)=(-(params(24)*params(18)));
  g1(7,4)=params(3)*params(17)-(-(params(26)*(params(2)-params(1))*params(5)));
  g1(7,7)=(-(params(3)*params(17)));
  g1(7,15)=params(26)*(params(2)-params(1))*params(5);
  g1(7,16)=(-(params(26)-params(26)*params(1)-params(26)*(params(2)-params(1))*params(5)));
  g1(7,19)=params(26)*(params(2)-params(1))*params(5);
  g1(7,22)=params(26)*(params(2)-params(1))*params(5);
  g1(8,4)=(-(params(2)-params(1)));
  g1(8,13)=(-params(2));
  g1(8,15)=params(1);
  g1(8,22)=(-(params(2)-params(1)));
  g1(9,3)=(-params(7));
  g1(9,4)=(-(1-params(7)));
  g1(9,5)=1;
  g1(10,6)=(-params(7));
  g1(10,7)=(-(1-params(7)));
  g1(11,1)=1;
  g1(11,8)=(-params(6));
  g1(11,9)=(-(1-params(6)));
  g1(11,17)=(-1);
  g1(12,1)=(-1);
  g1(12,2)=1;
  g1(12,8)=1;
  g1(12,10)=1;
  g1(13,1)=(-1);
  g1(13,2)=1;
  g1(13,9)=1;
  g1(13,11)=1;
  g1(14,2)=params(10);
  g1(14,15)=1-params(2);
  g1(15,14)=1-params(11);
  g1(15,15)=(-((1-params(11))*(1+params(12))));
  g1(15,18)=(-1);
  g1(16,13)=1;
  g1(16,14)=(-1);
  g1(17,13)=(-1);
  g1(17,15)=1;
  g1(17,20)=1;
  g1(18,1)=1;
  g1(18,5)=(-1);
  g1(19,17)=1-params(14);
  g1(20,18)=1;
  g1(21,1)=params(16);
  g1(21,12)=params(15);
  g1(21,19)=1;
  g1(22,21)=0.2;
  if ~isreal(g1)
    g1 = real(g1)+2*imag(g1);
  end
if nargout >= 3,
  %
  % Hessian matrix
  %

  g2 = sparse([],[],[],22,484);
if nargout >= 4,
  %
  % Third order derivatives
  %

  g3 = sparse([],[],[],22,10648);
end
end
end
end
