% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    NOTES   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This file solves the nonlinear model with long-term loan a la GKS 2014. 
% Fixed amortization rate
% In this version Ri is the effective interest rate on the debt stock and  
% Rf is the fixed rate for new loans
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----------------------------------------------------------------
% 0. Housekeeping
%----------------------------------------------------------------
close all;

warning off;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

% 1.1 Endogenous variables 
    
var LAMBDAp LAMBDAi C Cp Ci K INV H Hp Hi X Np Ni L Qh Qk R Ri Rf Rk Wp Wi RMC PIiw PIpw PI GAMAibc Y Upsilon D Omegap Omegai Up Ui Vp Vi V
    %new observed variables    
    dy_obs dc_obs dinv_obs dx_obs ffr_obs pi_obs dhp_obs spread_obs dcredit_obs credit2gdp %hrs_obs 
    %Auxilary variables
    auxi_Lambdap auxi_Lambdai auxi_LambdapPi auxi_LambdapQh auxi_LambdaiQh auxi_LambdaQk auxi_LambdaRck auxi_NKPCWp1 auxi_NKPCWp2 
    auxi_NKPCWi1 auxi_NKPCWi2 auxi_NKPCPI1 auxi_NKPCPI2 auxi_X1 auxi_X2 auxi_INV1 auxi_INV2
    auxi_Upsilon auxi_OmegapRf auxi_OmegaiRf auxi_Omegap auxi_Omegai auxi_GAMAmq auxi_LambdaiPi auxi_GAMAibc
           
M                    % monetary policy shock 
OMEc                 % consumption demand shock
OMEhd                % housing demand shock
OMEn                 % labour supply shock 
ISh                  % housing investment specific shock
ISk                  % capital investment specific shock
CPw                  % wage markup shock
CPph                 % domestric price markup shock
LVR                  % shock to LVR
S                    % government expenditure
Z                    % technology shock
; 

varexo 

em                  % innovation to monetary policy shock
ec                  % innovation to consumption demand shock
ehd                 % innovation to housing demand shock
en                  % innovation to labour supply shock
eah                 % innovation to housing investment specific shock
eak                 % innovation to capital investment specific shock
eph                 % innovation to domestric price markup shock
%elvr                % absolute deviation of LVR shock
eg                  % shock to government expenditure
ez                  %innovation to growth in technology shock
ew
eup
;


%----------------------------------------------------------------
% 2. Parameters of the model
%----------------------------------------------------------------

% 2.1 Parameters

parameters   sigma_n varsigma_c varsigma_n beta_p beta_i delta_h delta_k eta_k eta_l tauss xi_hi xi_hp xi_np xi_ni piss piwss;
parameters   iota_ph iota_w kappa_w kappa_ph kappa_k kappa_h eta_h eta_w rho_up kappa_hp kappa_hi kappa_ke rho_n rho_z;
parameters   lvrss vi eta_p eta_y eta_dy rho_m rho_c rho_hd rho_ah rho_ak rho_ph rho_g gbar rho_w rho_lvr eta_lvr;% czcap;

%m_i2 

%kappa alpha

% 2.2 Steady state variables
% parameters piss gdss tauss qhss qkss Rss rkss rmcss kss invss LSiss LSpss ldss lqhiss Upsilonss Riss xss;
% parameters gammaibcss phi2ss phi3ss phi4ss phi5ss phi6ss phi7ss ciss qhiss hiss lss dss phipss phihss cpss css;
% parameters hpss hss wp_2 wp_3 npss wi_2 wi_3 niss nss npnss ninss mupss muiss omegapss omegaiss;
 
%----------------------------------------------------------------
%                   4. Dynamics of the Model 
%----------------------------------------------------------------

model; %64 endogenous equations

%----------------------------   4.1 Patient Household   ------------------------------

    Up = exp(OMEc)*(log(exp(Cp) - varsigma_n*exp(Cp(-1))) + xi_hp*exp(OMEhd)*Hp - xi_np*exp(OMEn)*(exp(Np)^(1+sigma_n))/(1+sigma_n));
    
    Vp = Up + beta_p * Vp(+1);
    
    % LAMBDAp = OMEc - (1/(1-varsigma_c/gbar))*(Cp - (varsigma_c/gbar)*(Cp(-1) - gz));  
    % 1                        
     exp(LAMBDAp) = exp(OMEc)/(exp(Cp) - varsigma_c*exp(Cp(-1))); 
     
    % 0 = LAMBDAp(+1) - LAMBDAp  + R - gz(+1)- PI(+1);   
    % 2
     0 = log(beta_p) + R + auxi_LambdapPi(+1);   
    % 3
         auxi_Lambdap = LAMBDAp - LAMBDAp(-1);
    % 4
         auxi_LambdapPi = auxi_Lambdap - PI;
         
    % Qh  + kappa_hp*(Hp - Hp(-1)) = (1-beta_p*(1-delta_h)/gbar)*(OMEc + OMEhd - Hp - LAMBDAp) + (beta_p*(1-delta_h)/gbar)*(LAMBDAp(+1) - LAMBDAp + Qh(+1)- gz(+1) );
    % 5
      exp(Qh) = xi_hp*(exp(OMEc + OMEhd - Hp - LAMBDAp)) + beta_p*(1-delta_h)*(exp(auxi_LambdapQh(+1))) -kappa_hp*(exp(Hp) - exp(Hp(-1)))/exp(STEADY_STATE(Hp));     
    % 6    
         auxi_LambdapQh  = auxi_Lambdap + Qh;      
    
    % Qk + kappa_ke*(K - K(-1)) = (1-beta_p*(1-delta_k)/gbar)*(LAMBDAp(+1) - LAMBDAp + Rk(+1) - gz(+1)) + (beta_p*(1-delta_k)/gbar)*(LAMBDAp(+1) - LAMBDAp + Qk(+1)- gz(+1));
    % 7
      exp(Qk) = beta_p*exp(auxi_LambdaRck(+1)) + beta_p*(1-delta_k)*exp(auxi_LambdaQk(+1))- kappa_ke *(exp(K) - exp(K(-1)))/exp(STEADY_STATE(K));
    % 8
         auxi_LambdaQk  = auxi_Lambdap + Qk;
    % 9     
         auxi_LambdaRck  = auxi_Lambdap + Rk;
         
    % PIpw - iota_w*PIpw(-1) = beta_p*(PIpw(+1) - iota_w*PIpw) + ((eta_w-1)/kappa_w)*(CPw + OMEc + sigma_n*Np - varsigma_n*(1+sigma_n)*Np(-1) - LAMBDAp - Wp);  
    % 10                                                                                      
      1 =  -xi_np*exp(CPw + OMEc + OMEn + sigma_n*Np - LAMBDAp - Wp) + exp(CPw)
         + kappa_w*((exp(PIpw)/(exp(PIpw(-1))^iota_w)/(piwss^(1-iota_w)))^2 - exp(PIpw)/(exp(PIpw(-1))^iota_w)/(piwss^(1-iota_w))) - (1/2)*kappa_w*exp(CPw)*((exp(PIpw)/(exp(PIpw(-1))^iota_w)/(piwss^(1-iota_w)))-1)^2 
         - kappa_w*beta_p*(exp(auxi_NKPCWp1(+1)) - exp(auxi_NKPCWp2(+1)));  
    % 11    
         auxi_NKPCWp1  = auxi_LambdapPi + Np- Np(-1) + 3*PIpw - 2*iota_w*PIpw(-1) - 2*(1-iota_w)*STEADY_STATE(PIpw);
    % 12     
         auxi_NKPCWp2  = auxi_LambdapPi + Np- Np(-1) + 2*PIpw - iota_w*PIpw(-1) - (1-iota_w)*STEADY_STATE(PIpw);
        
    %Upsilonss*Upsilon  - omegapss*Riss*(Omegap + Rf) = (beta_p*(1-tauss)/piss/gbar)*(Upsilonss-omegapss*Riss)*(LAMBDAp(+1) - LAMBDAp  - PI(+1)- gz(+1)) 
    % 13                                                + (beta_p*(1-tauss)/piss/gbar)*(Upsilonss*Upsilon(+1)  - omegapss*Riss*(Omegap(+1) + Rf(+1)));
      1 + exp(Upsilon) - exp(Omegap + Rf) = beta_p*(1-tauss)*(exp(auxi_Upsilon(+1)) - exp(auxi_OmegapRf(+1)));
    % 14
           auxi_Upsilon = auxi_LambdapPi + Upsilon;
    % 15       
           auxi_OmegapRf  = auxi_LambdapPi + Omegap + Rf;
                                                                                                                                                              
    % omegapss*Omegap  = (beta_p*(1+omegapss*(1-tauss))/piss/gbar)*(LAMBDAp(+1) - LAMBDAp  - PI(+1)- gz(+1)) + (beta_p*(omegapss*(1-tauss))/piss/gbar)* Omegap(+1);
    % 16 
      exp(Omegap) = beta_p*exp(auxi_LambdapPi(+1))+ beta_p*(1-tauss)*exp(auxi_Omegap(+1));
    % 17          
           auxi_Omegap  = auxi_LambdapPi + Omegap;
    
    % D = ((1-tauss)/piss/gbar)*(D(-1) - PI - gz) + ldss*L;
    % 18
      exp(D) = (1-tauss)*exp(D(-1)-PI) + exp(L);
    
    % Ri = ( 1 - ldss )* Ri(-1) + ldss* Rf;
    % 19
      exp(Ri) = exp(Ri(-1)) - exp(L - D + Ri(-1)) +  exp(L - D + Rf);
                         
%----------------------------   4.3 Impatient Household   -----------------------------

    Ui = exp(OMEc)*(log(exp(Ci) - varsigma_n*exp(Ci(-1))) + xi_hi*exp(OMEhd)*Hi - xi_ni*exp(OMEn)*(exp(Ni)^(1+sigma_n))/(1+sigma_n));
    
    Vi = Ui + beta_i * Vi(+1);
        
    % LAMBDAi = OMEc - (1/(1-varsigma_c*(1/gbar)))*(Ci - (varsigma_c/gbar)*(Ci(-1)  - gz));
    % 20
      exp(LAMBDAi) = exp(OMEc)/(exp(Ci) - varsigma_c*exp(Ci(-1))); %6
    
    % Qh + kappa_hi*(Hi - Hi(-1)) - gammaibcss*lvrss*Qh - lvrss*GAMAibc  - gammaibcss*lvr = (1- gammaibcss*lvrss)*(1- beta_i*(1-delta_h)/gbar)*(OMEc + OMEhd - Hi - LAMBDAi) + 
    %                                                                 ((1-delta_h)*beta_i/gbar)*((1-gammaibcss*lvrss)*(LAMBDAi(+1) - LAMBDAi + Qh(+1)- gz(+1))- lvrss*GAMAibc(+1) - gammaibcss*lvr(+1));                                                                         
 
      exp(Qh) + exp(GAMAibc + LVR + Qh) = xi_hi*(exp(OMEc + OMEhd - Hi - LAMBDAi))+ beta_i*(1-delta_h)*(exp(auxi_LambdaiQh(+1)) + exp(auxi_GAMAmq(+1))) - kappa_hi*(exp(Hi) - exp(Hi(-1)))/exp(STEADY_STATE(Hi));% d
   
           auxi_Lambdai = LAMBDAi - LAMBDAi(-1);
           
           auxi_LambdaiQh  = auxi_Lambdai + Qh ;
         
           auxi_GAMAmq    =  auxi_Lambdai + Qh + LVR + GAMAibc;       
           
    % -gammaibcss*GAMAibc + omegaiss*Riss*(Omegai + Rf) = (beta_i*(1-tauss)/piss/gbar)*(omegaiss*Riss-gammaibcss)*(LAMBDAi(+1) - LAMBDAi  - PI(+1)- gz(+1)) 
    %                                                 + (beta_i*(1-tauss)/piss/gbar)*(omegaiss*Riss*(Omegai(+1) + Rf(+1)) - gammaibcss*GAMAibc(+1) );
    % 25                                                                                      
      1 + exp(GAMAibc) - exp(Omegai + Rf) = beta_i*(1-tauss)*(exp(auxi_GAMAibc(+1)) - exp(auxi_OmegaiRf(+1)));
      
           auxi_GAMAibc = auxi_LambdaiPi + GAMAibc;
           
           auxi_OmegaiRf  = auxi_LambdaiPi + Omegai + Rf;  
     
    % omegaiss*Omegai  = (beta_i*(omegaiss*(1-tauss)-1)/piss/gbar)*(LAMBDAi(+1) - LAMBDAi  - PI(+1)- gz(+1)) + (beta_i*(omegaiss*(1-tauss))/piss/gbar)* Omegai(+1); 
      % 28                                                                           
      exp(Omegai) = beta_i*exp(auxi_LambdaiPi(+1))+ beta_i*(1-tauss)*exp(auxi_Omegai(+1));
           
           auxi_LambdaiPi = auxi_Lambdai - PI;
                      
           auxi_Omegai  = auxi_LambdaiPi + Omegai;
    
    % lss*L = lvrss*hiss*( Qh + Hi ) - (lvrss*hiss*(1-delta_h)/gbar)*(Qh + Hi(-1)- gz )+ hiss*LVR - (hiss*(1-delta_h)/gbar)*LVR  ;
    % 31
      exp(L)  =  exp(LVR + Qh + Hi ) - (1-delta_h)*exp(LVR + Qh + Hi(-1) );
    
    % ciss*Ci = LSiss*(Wi + Ni) + lss*L - hiss*(Qh + Hi)+ (hiss*(1-delta_h)/gbar)*(Qh + Hi(-1) - gz) - (Riss*dss/piss/gbar)*(Ri(-1) + D(-1) - PI - gz) + ((1-tauss)*dss/piss/gbar)*(D(-1) - PI - gz);             
    % 32
      exp(Ci) + exp(Qh + Hi) - (1-delta_h)*exp(Qh +Hi(-1)) + exp(D(-1) + Ri(-1) - PI ) - (1-tauss)*exp(D(-1) - PI ) = exp(Wi + Ni )+ exp(L) - exp(Ni)*((1/2)*kappa_w*exp(Wi)*(((PIpw+1)/((PIpw(-1)+1)^iota_w)/(piwss^(1-iota_w)))-1)^2); %12

    % PIiw - iota_w*PIiw(-1) = beta_i*(PIiw(+1) - iota_w*PIiw) + ((eta_w-1)/kappa_w)*(CPw + OMEc + sigma_n*Ni - varsigma_n*(1+sigma_n)*Ni(-1) - LAMBDAi - Wi);
    % 33
      1 =  -xi_ni*exp(CPw + OMEc + OMEn + sigma_n*Ni - LAMBDAi - Wi) + exp(CPw)
           + kappa_w*((exp(PIiw)/(exp(PIiw(-1))^iota_w)/(piwss^(1-iota_w)))^2 - exp(PIiw)/(exp(PIiw(-1))^iota_w)/(piwss^(1-iota_w))) - (1/2)*kappa_w*exp(CPw)*((exp(PIiw)/(exp(PIiw(-1))^iota_w)/(piwss^(1-iota_w)))-1)^2
           - kappa_w*beta_i*(exp(auxi_NKPCWi1(+1)) - exp(auxi_NKPCWi2(+1)));  
             
           auxi_NKPCWi1  = auxi_Lambdai + Ni- Ni(-1) -PI + 3*PIiw - 2*iota_w*PIiw(-1) - 2*(1-iota_w)*STEADY_STATE(PIiw);
         
           auxi_NKPCWi2  = auxi_Lambdai + Ni- Ni(-1) -PI + 2*PIiw - iota_w*PIiw(-1) - (1-iota_w)*STEADY_STATE(PIiw);

%----------------------------     4.4   Firms     -----------------------------

    % Wp = K(-1) + Rk - gz - Np;
    % 36
      Wp = log((eta_l)*(1-eta_k)) + Y + RMC  - Np ;
      
    % Wi = K(-1) + Rk - gz - Ni;
    % 37
      Wi = log((1 - eta_l)*(1-eta_k)) + Y + RMC  - Ni;

    % Y = eta_k * (K(-1) - gz) + (1-eta_k)*(eta_l *Np + (1-eta_l)*Ni);
    % 38
      Y = Z + eta_k * K(-1) + (1-eta_k)*(eta_l *Np + (1-eta_l)*Ni);

    % RMC = eta_k*Rk + (1-eta_k)*(eta_l *Wp + (1-eta_l)*Wi);
    
      %RMC = eta_k*Rk + (1-eta_k)*(eta_l *Wp + (1-eta_l)*Wi) - z - eta_k*log(eta_k) - eta_l*(1-eta_k)*(log(eta_l)+log(1-eta_k))- (1 - eta_l)*(1-eta_k)*(log(1 - eta_l)+log(1-eta_k));
    % 39
      Rk  = log(eta_k) + Y + RMC - K(-1);

    % PI - iota_ph*PI(-1) = beta_p*(PI(+1) - iota_ph*PI) + ((eta_h-1)/kappa_ph)*(CPph + RMC);
    % 40
      0 =  1 - exp(CPph) - kappa_ph*((exp(PI)/(exp(PI(-1))^iota_ph)/(piss^(1-iota_ph)))^2 - exp(PI)/(exp(PI(-1))^iota_ph)/(piss^(1-iota_ph))) + exp(CPph + RMC)
           + (1/2)*kappa_ph*exp(CPph)*((exp(PI)/(exp(PI(-1))^iota_ph)/(piss^(1-iota_ph)))-1)^2 
           + kappa_ph*beta_p*(exp(auxi_NKPCPI1(+1)) - exp(auxi_NKPCPI2(+1)));  
         
             auxi_NKPCPI1  = auxi_Lambdap + Y- Y(-1) + 2*PI - 2*iota_w*PI(-1) - 2*(1-iota_w)*STEADY_STATE(PI);
         
             auxi_NKPCPI2  = auxi_Lambdap + Y- Y(-1) + PI - iota_w*PI(-1) - (1-iota_w)*STEADY_STATE(PI);
         
%----------------------------   4.5  Final Goods Firms     -----------------------------

    % X = (1/(1+beta_p))*( X(-1)  - gz ) + (beta_p/(1+beta_p))*( X(+1) + gz(+1) ) +  (1/(1+beta_p))*(1/((gbar^2)*kappa_h))*(Qh + ISh) ;
    
      exp( - Qh - ISh) = 1 - (1/2)*kappa_h*(exp(X-X(-1)) - 1)^2 - kappa_h*(exp(2*X-2*X(-1)) - exp(X-X(-1))) + beta_p*kappa_h*(exp(auxi_X1(+1)) - exp(auxi_X2(+1)));
                                                    
         auxi_X1  = auxi_Lambdap + ISh- ISh(-1) + Qh- Qh(-1) + 3*X - 3*X(-1);
         
         auxi_X2  = auxi_Lambdap + ISh- ISh(-1) + Qh- Qh(-1) + 2*X - 2*X(-1);

    % H = (1-delta_h)*(1/gbar)*(H(-1) - gz) + (1 - (1-delta_h)*(1/gbar))*(ISh + X); %19
    
      exp(H) = (1-delta_h)*exp(H(-1)) + exp(ISh + X)*( 1 - (1/2)*kappa_h*(exp(X-X(-1)) - 1)^2 );

    % INV = (1/(1+beta_p))*( INV(-1)  - gz ) + (beta_p/(1+beta_p))*( INV(+1) + gz(+1) ) +  (1/(1+beta_p))*(1/((gbar^2)*kappa_k))*(Qk + ISk) ;
    
      exp( - Qk - ISk) = 1 - (1/2)*kappa_k*(exp(INV-INV(-1)) - 1)^2 - kappa_k*(exp(2*INV-2*INV(-1)) - exp(INV-INV(-1))) + beta_p*kappa_k*(exp(auxi_INV1(+1)) - exp(auxi_INV2(+1)));
                                                 
         auxi_INV1  = auxi_Lambdap + ISk- ISk(-1) + Qk- Qk(-1) + 3*INV - 3*INV(-1);
         
         auxi_INV2  = auxi_Lambdap + ISk- ISk(-1) + Qk- Qk(-1) + 2*INV - 2*INV(-1);

    % K = (1-delta_k)*(1/gbar)*(K(-1) - gz) + (1 - (1-delta_k)*(1/gbar))*(ISk + INV);

      exp(K) = (1-delta_k)*exp(K(-1)) + exp(ISk + INV)*( 1 - (1/2)*kappa_k*(exp(INV-INV(-1)) - 1)^2 );

%----------------------------  4.6 Central Bank  -----------------------------

    R = vi*R(-1) + (1-vi)*(STEADY_STATE(R) + eta_p*(PI - STEADY_STATE(PI) )+ eta_y*(Y - Y(-1))) + M; %39
    
    LVR = rho_lvr*LVR(-1) + (1-rho_lvr)*STEADY_STATE(LVR) + (1-rho_lvr)*eta_lvr*(Qh-STEADY_STATE(Qh));
    
    % , D - D(-1), credit2gdp - STEADY_STATE(credit2gdp)

%---------------------------- 4.7 Market Clearing  -----------------------------

    % Y = css*C + xss*X + invss*INV + gdss*S ; %26
    % 53
      exp(Y)= exp(C) + exp(X) + exp(INV) + exp(S);

    % css*C = cpss*Cp + ciss*Ci;
    % 54
      exp(C) = exp(Cp) + exp(Ci);
   
    % hss*H = hpss*Hp + hiss*Hi;  % 28
    % 55
      exp(H) = exp(Hp) + exp(Hi);

%----------------------------  4.8 Relative Prices  -----------------------------
    % 56
    PIiw = Wi - Wi(-1) + PI;
    % 57
    PIpw = Wp - Wp(-1) + PI;

%-------------------------------- 4.10 shocks block --------------------------------------

    M    = rho_m* M(-1) + em;    % Monetary policy shock

    OMEc  = rho_c*OMEc(-1) + ec;  
    
    OMEn  = rho_n*OMEn(-1) + en;

    OMEhd = rho_hd*OMEhd(-1) + ehd;         

    ISh   = rho_ah* ISh(-1) + eah;

    ISk   = rho_ak* ISk(-1) + eak;

    CPw   = (1-rho_w)*log(eta_w) + rho_w* CPw(-1) + ew;
 
    CPph  = (1-rho_ph)*log(eta_h) + rho_ph* CPph(-1) + eph;

    Z     = rho_z* Z(-1) + ez; 

    %LVR      = rho_lvr* LVR(-1) + elvr;  % shock to monitoring costs
    % 67
    S        = (1-rho_g)*STEADY_STATE(S) + rho_g*S(-1) + eg;

    Upsilon - STEADY_STATE(Upsilon) = rho_up*(Upsilon(-1) - STEADY_STATE(Upsilon)) + eup; 
    

%-------------------------------- 4.11 observables --------------------------------------

   %41. Output
   	dy_obs = Y - Y(-1) ;

    %dy_sim = dy + 100*log(gbar);

    %dy_sim = dy_obs + log(gbar);

   %42. Consumption 
   	dc_obs = C - C(-1) ;

   %43. Investment 
   	dinv_obs = INV - INV(-1) ;
		
   %44. Residential Investment 
    dx_obs = X - X(-1) ;

   %45. Federal Funds
    ffr_obs = 4*R;

   %46.  Inflation rate
    pi_obs = PI;

   %46. Assume observable is real house price growth
    dhp_obs = Qh - Qh(-1);			

   %47. Hours
	%hrs_obs = Np*npnss + Ni*ninss;   
	
   %48. Wages
   	%define aggregate nominal wage inflation
	%dw = eta_l*PIpw + (1-eta_l)*PIiw + (eta_l - npnss)*(Np-Np(-1)) + ((1-eta_l) - ninss)*(Ni-Ni(-1));
    
    %credit

	dcredit_obs = D - D(-1);
    
    credit2gdp = D - Y;
   
    %Due to new definiton of Ri as the effective rate, we instead match the
    %obsrved spread with Rf
    spread_obs = 4*(Rf - R);
    
    V = (1-beta_p)*Vp + (1-beta_i)*Vi;
     
end;
%----------------------------------------------------------------
% 5. Computation
%----------------------------------------------------------------

% steady;
% check;

% 5.1 Distribution of shocks

shocks;
var em;     stderr 0.0012;      % innovation to monetary policy shock  
var ec;     stderr 0.0368;      % innovation to monetary policy shock  
 var ehd;    stderr 0.1033;      % innovation to housing demand shock
 var eah;    stderr 0.0133;      % innovation to housing investment specific shock     
var eak;    stderr 0.0740;      % innovation to housing investment specific shock   
var eph;    stderr 0.0230;      % innovation to domestric price markup shock
var eup;    stderr 0.1276;      % innovation to capital requirement shock (lending spread shock)
var eg;     stderr 0.0204;      % shock to government expenditure
var ez;     stderr 0.007;      % innovation to domestric price markup shock
var ew;     stderr 0.1844;      % innovation to domestric price markup shock
end;

% 5.2 Stochastic simulations

% stoch_simul(order=2,irf=0 , print, nograph) V Vp Vi Y C Cp Ci Hp Hi D Qh Np Ni; %R RESIDENTIAL_LOAN Qh Ci Hi Hp LBound Y PI

xparams = [-10:0.1:0];

%(2) rho_lvr
%let yparams be the smoothing parameter in the lvr policy rule
%yparams = linspace(0.75,0.99,2);
yparams = [0.75];

%(3) create the matrices of the values to be considered
[Xgrid,Ygrid] = meshgrid(xparams,yparams);
%Xgrid and Ygrid are the same size. They are maxcountery*maxcounterx


%(4) The number of x values to be considred
maxcounterx =size(xparams,2);

%(5) The number of y values to be considered
maxcountery = size(yparams,2);

nvar = 3; %number of simulated variables
%nperiods =100000; %number of periods data is simulated for. Must be consistent with the mod file
% we use the theoretical moments, rather than simulating data

%matrix to store the output
%simuldata = zeros(nperiods,maxcountery,maxcounterx); %each page is for a given x

stochmean = zeros(maxcountery,maxcounterx,nvar); %This saves the stochastic mean of the simulated series
stochvariance = zeros(maxcountery,maxcounterx,nvar); %This saves the stochastic variances of the simulated series

counter = 1;
num_mx=1;
num_my = 1;

 for num_mx = 1:maxcounterx %loop over x first
     
     for num_my = 1:maxcountery %loop over y for a given x
         
         eta_lvr = Xgrid(num_my,num_mx);
         rho_lvr = Ygrid(num_my,num_mx);
         save parameterfile.mat rho_lvr eta_lvr
         stoch_simul(order=2,irf=0, noprint, nograph) V Vp Vi Y C Cp Ci Hp Hi D Qh Np Ni;
         %axa = [dy_obs,dc_obs,dinv_obs,dx_obs,ffr_obs,pi_obs,dhp_obs,hrs_obs,spread_obs,dy_sim];
         %simuldata(:,num_my,num_mx) = dy_sim; %x indexes the pages
         
        %save the variances of all of the simulated series (5)
        %dlvr Y D Qh pi_obs
        
        for jj=1:nvar
        
            stochmean(num_my,num_mx,jj) = oo_.mean(jj);
            stochvariance(num_my,num_mx,jj) = oo_.var(jj,jj);
        end
       counter = counter + 1;
     end
 end
 
save variancesout stochmean stochvariance nvar
save grids Xgrid Ygrid

%% plot

figure() % Volatility

subplot(1,3,1)  
    plot(Xgrid(1:maxcounterx),stochmean(1,1:maxcounterx,1)*100,'-b','LineWidth', 2.0);
        xlabel('Policy response','FontSize',12);
    title('Social Welfare','FontSize',14,'FontWeight','bold');
   % ylabel('\bf%','FontSize',12); 
    %legend( 'Output','Credit');
    %legend('Zeta = 0.5', 'Zeta = 1.5', '','FontSize',14);
    %xlim([1 30])
    %hold off
subplot(1,3,2)  
    plot(Xgrid(1:maxcounterx),stochmean(1,1:maxcounterx,2)*100,'-b','LineWidth', 2.0);
        xlabel('Policy response','FontSize',12);
%ylabel('\bf%','FontSize',12); 
    title('Saver Welfare','FontSize',14,'FontWeight','bold');
subplot(1,3,3)  
    plot(Xgrid(1:maxcounterx),stochmean(1,1:maxcounterx,3)*100,'-b','LineWidth', 2.0);
        xlabel('Policy response','FontSize',12);
%ylabel('\bf%','FontSize',12); 
    title('Borrower Welfare','FontSize',14,'FontWeight','bold');    
