% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    NOTES   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% This file solves optimal simple LTV rule with Iacoviello (2005) model
% without occasionally binding ZLB
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----------------------------------------------------------------
% 0. Housekeeping
%----------------------------------------------------------------
close all;

warning off;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

% 1.1 Endogenous variables 
    
var y x cu cc c hu hc lu  lc wu wc  bc r  dp q a_a r_e kt;  


//% THE GUYS BELOW ARE THE IID OF THE SHOCKS

varexo  eps_u eps_e;



//% PARAMETERS ENTER THE MODEL EQUATIONS


parameters BETATILDE, BETA, J, ETA, K, GAMMA, ALPHA, XSS, KTILDE, RHO, PHIP, PHIY, TECH phib rho_ltv; 
parameters CU_QHU PHI CC_QHC CC_Y CU_Y QHC_Y QHU_Y RSS;


BETA = 0.995 ;
BETATILDE = 0.985;
J = .1 ;
ETA = 2;//1.01 ; 
K = 0.9 ; % LTV ratio
GAMMA = .64; 
ALPHA = 0 ; 
XSS = 1.2 ; 
KTILDE =0.0858; 
RHO = 0.;
PHIP = 0.5 ; 
PHIY = 0.;
TECH=.9;
%phib=0.01;

STDERR_AU = 0.01; 

// STEADY STATE

CU_QHU = (1-BETA)/J;

PHI = (1-BETATILDE - K*(BETA - BETATILDE));

CC_QHC = PHI/J;

CC_Y = ((1-GAMMA)/XSS)*(PHI/(PHI+J*K*(1-BETA)));

CU_Y = 1 - CC_Y;

QHC_Y = CC_Y/CC_QHC;

QHU_Y = (XSS*J*(PHI+J*K*(1-BETA)) - J*(1-GAMMA)*PHI)/(XSS*(PHI+J*K*(1-BETA))*(1-BETA));

RSS = 1/BETA;

no_irfs_plot  = 0 ;

DO_IRFS       = 1 ;

  load parameterfile;
  set_param_value('phib',phib);
  set_param_value('rho_ltv',rho_ltv);
  
  weight_y = 0;
  
  weight_ltv = 10;

//%------------------------------------------------------------
//% Model equations
//%------------------------------------------------------------

model;

%----------------------------   4.1 Patient Household   ------------------------------

//% eq 1
-cu = r - dp(+1)-cu(+1) ;

//% eq 2
wu = (ETA-1)*lu + cu ;

//% eq 6
wc = (ETA-1)*lc + cc ;

//% eq 3
hu = (1/(1-BETA))*(cu-q) - (BETA/(1-BETA))*(cu(+1)-q(+1)) ;

//% eq 5
bc = kt + q(+1) + hc + dp(+1) - r(+1);

//% eq 4
cc = ((PHI+J*K*(1-BETA))/PHI)*(y - x) - J/PHI*(hc-hc(-1))  + (K*J/PHI)*(BETA*bc - bc(-1))  - (K*J)*(r(-1)-dp) ;

//% eq 7
hc = ((1-K*BETA)/PHI)*cc - (1/PHI)*q + (K*BETA/PHI)*( kt - r + dp(+1)) + ((BETATILDE/PHI)+(K/PHI)*(BETA-BETATILDE))*q(+1)  - (BETATILDE*(1-K)/PHI)*cc(+1) - (BETATILDE*K/PHI)*kt;

//% eq 8
c = (CC_Y)*cc + (CU_Y)*cu ;

//% eq 10
0= (QHU_Y)*hu + (QHC_Y)*hc ;

//% eq 11
y =a_a + GAMMA*lu + (1-GAMMA)*lc ;

//% eq 12
wu =  y - x - lu ;

//% eq 13
wc =  y - x - lc ;

//% eq 14
dp = BETA*dp(+1) - KTILDE*x ;

//% eq 15
r = RHO*r(-1) + (1-RHO)*(1+PHIP)*dp + (1-RHO)*PHIY*(y-y(-1)) + r_e   ;

%r = 1 -  RSS ;

//% eq 16
y = c ;

a_a=TECH*a_a(-1)+eps_u;

r_e=0.0*r_e(-1)+eps_e;

kt= rho_ltv*kt(-1) - phib*bc;%(bc - y);%(bc - bc(-1));

end ;



//% CALCULATIONS OF STEADY STATE

%steady(solve_algo=2) ;

check;

//% STOCHASTIC PROPERTIES OF SHOCKS

shocks;

var eps_u; stderr 0.03;
var eps_e; stderr 0;
end;
% 5.2 Stochastic simulations

% stoch_simul(order=2,irf=0 , print, nograph) V Vp Vi Y C Cp Ci Hp Hi D Qh Np Ni; %R RESIDENTIAL_LOAN Qh Ci Hi Hp LBound Y PI

xparams = [0:0.01:0.9];

%(2) rho_lvr
%let yparams be the smoothing parameter in the lvr policy rule
%yparams = linspace(0.75,0.99,2);
yparams = [0:0.1:0.9];

%(3) create the matrices of the values to be considered
[Xgrid,Ygrid] = meshgrid(xparams,yparams);
%Xgrid and Ygrid are the same size. They are maxcountery*maxcounterx


%(4) The number of x values to be considred
maxcounterx =size(xparams,2);

%(5) The number of y values to be considered
maxcountery = size(yparams,2);

nvar = 3; %number of simulated variables
%nperiods =100000; %number of periods data is simulated for. Must be consistent with the mod file
% we use the theoretical moments, rather than simulating data

%matrix to store the output
%simuldata = zeros(nperiods,maxcountery,maxcounterx); %each page is for a given x

%stochmean = zeros(maxcountery,maxcounterx,nvar); %This saves the stochastic mean of the simulated series
stochvariance = zeros(maxcountery,maxcounterx,nvar); %This saves the stochastic variances of the simulated series
LOSS_zlb = zeros(maxcountery,maxcounterx);

counter = 1;
num_mx=1;
num_my = 1;

 for num_mx = 1:maxcounterx %loop over x first
     
     for num_my = 1:maxcountery %loop over y for a given x
         
         phib = Xgrid(num_my,num_mx);
         rho_ltv = Ygrid(num_my,num_mx);
         save parameterfile.mat phib rho_ltv;
         stoch_simul(order=1,irf=0, noprint, nograph)bc y kt;
         %axa = [dy_obs,dc_obs,dinv_obs,dx_obs,ffr_obs,pi_obs,dhp_obs,hrs_obs,spread_obs,dy_sim];
         %simuldata(:,num_my,num_mx) = dy_sim; %x indexes the pages
         
        %save the variances of all of the simulated series (5)
        %dlvr Y D Qh pi_obs
        
        for jj=1:nvar        
            %stochmean(num_my,num_mx,jj) = oo_.mean(jj);
            stochvariance(num_my,num_mx,jj) = oo_.var(jj,jj);
        end
        
        LOSS_ltv(num_my,num_mx) = oo_.var(1,1) + weight_y* oo_.var(2,2) + weight_ltv*oo_.var(3,3);
        
        counter = counter + 1
     end
 end
 
save variancesout LOSS_ltv stochvariance nvar %stochmean 
save grids Xgrid Ygrid

%% plot

% figure() % Volatility
% 
% subplot(2,2,2)  
%     plot(Xgrid(1:maxcounterx),stochvariance(1,1:maxcounterx,1)*100,'-b','LineWidth', 2.0);
%         xlabel('Policy response','FontSize',12);
%     title('Volatility of Debt','FontSize',14,'FontWeight','bold');
%    % ylabel('\bf%','FontSize',12); 
%     %legend( 'Output','Credit');
%     %legend('Zeta = 0.5', 'Zeta = 1.5', '','FontSize',14);
%     %xlim([1 30])
%     %hold off
% subplot(2,2,3)  
%     plot(Xgrid(1:maxcounterx),stochvariance(1,1:maxcounterx,2)*100,'-b','LineWidth', 2.0);
%         xlabel('Policy response','FontSize',12);
% ylabel('\bf%','FontSize',12); 
%     title('Volatility of Output','FontSize',14,'FontWeight','bold');
% subplot(2,2,4)  
%     plot(Xgrid(1:maxcounterx),stochvariance(1,1:maxcounterx,3)*100,'-b','LineWidth', 2.0);
%         xlabel('Policy response','FontSize',12);
% %ylabel('\bf%','FontSize',12); 
%     title('Volatility of LTV Ratio','FontSize',14,'FontWeight','bold');    
%     
% subplot(2,2,1)  
%     plot(Xgrid(1:maxcounterx),LOSS_ltv(1,1:maxcounterx)*100,'-b','LineWidth', 2.0);
%         xlabel('Policy response','FontSize',12);
% %ylabel('\bf%','FontSize',12); 
%     title('Total Loss phib','FontSize',14,'FontWeight','bold');    
    
figure()

surf(Xgrid,Ygrid,LOSS_ltv*100 );shading interp
title('\bfLoss Function','FontSize',12)
xlabel('Policy response to debt','FontSize',10);
ylabel('Policy smoothing','FontSize',10);

 [valmax, etapmax] =max(-LOSS_ltv')
 [valmaxmax,etaymax ] =max(valmax)
 LOSS_ltv(etapmax(etaymax),etaymax)
 xparams(etapmax(etaymax))
 yparams(etaymax)
