%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_difference holds the piece-wise linear solution for
%  VariableName.  VariableName_uncdifference holds the linear solution for
%  VariableName.


clear


global M_ oo_

phib = 0;
phiy = 0;
save parameterfile.mat phib phiy;

modnam = 'ico';
modnamstar = 'ico_zlb';

constraint = 'r<1 - RSS';
constraint_relax ='r>1 - RSS';

% Pick innovation for IRFs
irfshock =char('eps_u');      % label for innovation for IRFs'eps_e',

maxiter = 50;
tol0 = 1e-8;

nperiods = 30;

% % For IRF
SHOCKS = zeros(nperiods,1);

SHOCKS(2)= randn(1,1) * .25;

% For simulation
% SHOCKS= randn(nperiods,1) * .03;



 shockssequence = SHOCKS;

% Solve model, generate model IRFs
[zdatalinear zdatapiecewise zdatass oobase_ Mbase_  ] = ...
  solve_one_constraint(modnam,modnamstar,...
  constraint, constraint_relax,...
  shockssequence,irfshock,nperiods);


% unpack the IRFs                          
for i=1:M_.endo_nbr
  eval([deblank(M_.endo_names(i,:)),'_u=zdatalinear(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
  eval([deblank(M_.endo_names(i,:)),'_ss=zdatass(i);']);
end


%% Modify to plot IRFs and decision rules
%paramfile_fv
%load results2
% figure()
% 
% subplot(2,2,1)
% plot( 100*y_p(78:477),'k','LineWidth', 2)
%  hold on
%  %plot( 100*y_p_4(78:477),'--','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfOutput','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,400])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  legend('Occbin (2% interest rate)','Occbin (4% interest rate)')
%  
% subplot(2,2,2)
% plot( 100*bc_p(78:477),'k','LineWidth', 2)
%  hold on
% % plot( 100*bc_p_4(78:477),'--','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfDebt','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,400])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  %legend('Occbin','no ZLB')
%  
% subplot(2,2,3)
%  plot( 100*q_p(78:477),'k','LineWidth', 2)
%  hold on
%  %plot( 100*q_p_4(78:477),'--','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse Price','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,400]) 
% % 
% subplot(2,2,4)
% plot((r_p(78:477)+(RSS-1))*400,'k','LineWidth', 2)
% hold on
% %plot((r_p_4(78:477)+(1.0101-1))*400,'--','LineWidth', 2)
% title('Policy rate')
% ylabel('Annualized Level, PPt')
%  xlim([0,400])
% % 
%  subplot(3,2,5)
%  plot(100*hu_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*hu_u,'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse (Savers)','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% % subplot(3,2,6)
% %  plot( 100*hc_p,'r','LineWidth', 2)
% %  hold on
% %  plot( 100*hc_u,'--b','LineWidth', 2)
% %  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
% %  title('\bfHouse (Borrowers)','FontSize',12)
% %  ylabel('\bf% from S.S.')
% %  %csPatches(gcf,bindingdates)
% %  xlim([0,nperiods]) 
%  
% 
% %%
% figure()
% 
% subplot(3,2,1)
% plot( 100*bc_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*bc_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfDebt','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  legend('Occbin','no ZLB')
%  
%  subplot(3,2,2)
%  plot( 100*cc_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*cc_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfConsumption (Borrowers)','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% subplot(3,2,3)
%  plot(100*lu_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*lu_u,'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfLabor (Savers)','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,4)
%  plot( 100*lc_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*lc_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfLabor (Borrowers)','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% subplot(3,2,5)
%  plot(100*wu_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*wu_u,'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfWager (Savers)','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,6)
%  plot( 100*wc_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*wc_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfWage (Borrowers)','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% figure()
% 
% subplot(3,2,1)
% plot( 100*y_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*y_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfOutput','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  legend('Occbin','no ZLB')
% 
% subplot(3,2,2)
%  plot(100*dp_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*dp_u,'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfInflation','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,3)
%  plot( 100*q_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*q_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse Price','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% subplot(3,2,4)
% plot((r_p+(RSS-1))*400,'r','LineWidth', 2)
% hold on
% plot((r_u+(RSS-1))*400,'--b','LineWidth', 2)
% title('Policy rate')
% ylabel('Annualized Level, PPt')
%  xlim([0,nperiods])
% 
%  subplot(3,2,5)
%  plot(100*hu_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*hu_u,'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse (Savers)','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,6)
%  plot( 100*hc_p,'r','LineWidth', 2)
%  hold on
%  plot( 100*hc_u,'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse (Borrowers)','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
%%
figure() % figure 1 for paper

subplot(4,2,2)
plot( 100*y_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*y_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfOutput','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %ylim([0,3])
 %csPatches(gcf,bindingdates)
 legend('Occbin','Standard method')

subplot(4,2,3)
 plot(100*dp_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*dp_u(2:end),'--b','LineWidth', 2)
% plot( L_u,'-k','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfInflation','FontSize',12')
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %csPatches(gcf,bindingdates)
 
subplot(4,2,4)
 plot( 100*q_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*q_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfHouse Price','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([1,nperiods]) 

subplot(4,2,1)
plot((r_p(1:end)+(RSS-1))*400,'r','LineWidth', 2)
hold on
plot((r_u(1:end)+(RSS-1))*400,'--b','LineWidth', 2)
plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
title('Policy rate')
ylabel('Annualized Level, PPt')
 xlim([1,nperiods])
 
%   subplot(4,2,5)
%  plot(100*hu_p(2:end),'r','LineWidth', 2)
%  hold on
%  plot( 100*hu_u(2:end),'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse (Savers)','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
 
subplot(4,2,6)
 plot( 100*hc_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*hc_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfHouse (Borrowers)','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([1,nperiods]) 

 
subplot(4,2,5)
plot( 100*bc_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*bc_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfDebt','FontSize',12)
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %ylim([0,3])
 %csPatches(gcf,bindingdates)
%  %legend('Occbin','no ZLB')
%  
%  subplot(4,2,6)
% plot((kt_p(2:end)+1)*K*100,'r','LineWidth', 2)
% hold on
% plot((kt_u(2:end)+1)*K*100,'--b','LineWidth', 2)
% title('LTV Ratio')
% ylabel('Level in PPt')
%  xlim([0,nperiods])
 
 subplot(4,2,7)
 plot(100*lu_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*lu_u(2:end),'--b','LineWidth', 2)
% plot( L_u,'-k','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfLabor (Savers)','FontSize',12')
 ylabel('\bf% from S.S.')
 xlim([1,nperiods])
 %csPatches(gcf,bindingdates)
 
subplot(4,2,8)
 plot( 100*lc_p(2:end),'r','LineWidth', 2)
 hold on
 plot( 100*lc_u(2:end),'--b','LineWidth', 2)
 plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
 title('\bfLabor (Borrowers)','FontSize',12)
 ylabel('\bf% from S.S.')
 %csPatches(gcf,bindingdates)
 xlim([1,nperiods]) 
%% figure 2 in paper

% %  save results y_p dp_p q_p r_p bc_p kt_p
% % clear
% % save results y_p_ltv dp_p_ltv q_p_ltv r_p_ltv bc_p_ltv kt_p_ltv
% load results
% 
% figure() % figure for paper
% 
% subplot(3,2,3)
% plot( 100*y_p,'k','LineWidth', 1.5)
%  hold on
%  plot( 100*y_p_ltv,'--','LineWidth', 1.5)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfOutput','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  legend('No LTV','LTV')
% 
% subplot(3,2,4)
%  plot(100*dp_p,'k','LineWidth', 1.5)
%  hold on
%  plot( 100*dp_p_ltv,'--','LineWidth', 1.5)
% % plot( L_p_ltv,'-k','LineWidth', 1)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfInflation','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,5)
%  plot( 100*q_p,'k','LineWidth', 1.5)
%  hold on
%  plot( 100*q_p_ltv,'--','LineWidth', 1.5)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse Price','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% subplot(3,2,1)
% plot((r_p+(RSS-1))*400,'k','LineWidth', 1.5)
% hold on
% plot((r_p_ltv+(RSS-1))*400,'--','LineWidth', 1.5)
% title('Policy rate')
% ylabel('Annualized Level, PPt')
%  xlim([0,nperiods])
%  
% subplot(3,2,6)
% plot( 100*bc_p,'k','LineWidth', 1.5)
%  hold on
%  plot( 100*bc_p_ltv,'--','LineWidth', 1.5)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfDebt','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  %legend('Occbin','no ZLB')
%  
%  subplot(3,2,2)
% plot((kt_p+1)*K*100,'k','LineWidth', 1.5)
% hold on
% plot((kt_p_ltv+1)*K*100,'--','LineWidth', 1.5)
% title('LTV Ratio')
% ylabel('Level in PPt')
%  xlim([0,nperiods])

%% figure 3 in paper

% figure() % figure for paper
% 
% subplot(3,2,1)
% plot((r_p(1:end)+(RSS-1))*400,'r','LineWidth', 2)
% hold on
% plot((r_u(1:end)+(RSS-1))*400,'--b','LineWidth', 2)
% title('Policy rate')
% ylabel('Annualized Level, PPt')
%  xlim([0,nperiods])
%  
%  subplot(3,2,2)
% plot((kt_p(1:end)+1)*K,'r','LineWidth', 2)
% hold on
% plot((kt_u(1:end)+1)*K,'--b','LineWidth', 2)
% title('LTV Ratio')
% ylabel('Level in PPt')
%  xlim([0,nperiods])
% 
% subplot(3,2,3)
% plot( 100*y_p(2:end),'r','LineWidth', 2)
%  hold on
%  plot( 100*y_u(2:end),'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfOutput','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  %ylim([0,3])
%  %csPatches(gcf,bindingdates)
%  legend('Occbin','no ZLB')
% 
% subplot(3,2,4)
%  plot(100*dp_p(2:end),'r','LineWidth', 2)
%  hold on
%  plot( 100*dp_u(2:end),'--b','LineWidth', 2)
% % plot( L_u,'-k','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfInflation','FontSize',12')
%  ylabel('\bf% from S.S.')
%  xlim([0,nperiods])
%  %csPatches(gcf,bindingdates)
%  
% subplot(3,2,5)
%  plot( 100*q_p(2:end),'r','LineWidth', 2)
%  hold on
%  plot( 100*q_u(2:end),'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfHouse Price','FontSize',12)
%  ylabel('\bf% from S.S.')
%  %csPatches(gcf,bindingdates)
%  xlim([0,nperiods]) 
% 
% 
% subplot(3,2,6)
% plot( 100*bc_p(2:end),'r','LineWidth', 2)
%  hold on
%  plot( 100*bc_u(2:end),'--b','LineWidth', 2)
%  plot( zeros(nperiods+1,1),'-k','LineWidth', 1);
%  title('\bfDebt','FontSize',12)
%  ylabel('\bf% from S.S.')
%  xlim([1,nperiods])
%  