%% set inputs for solution below
%  The program produces responses to the shocks selected below under
%  irfshock. Paths for all the endogenous variables in the model selected
%  are produced. VariableName_difference holds the piece-wise linear solution for
%  VariableName.  VariableName_uncdifference holds the linear solution for
%  VariableName.
clear all;

global M_ oo_

% 2.3 Parameter values
 beta_p      = 0.993;                  % discount factor of patient household             (real risk-free rate)
 beta_b      = 0.9402;                  % discount factor of bank  This value target m_b = 0.89                         (funding spread for banks)
 beta_i      = 0.985;                  % discount factor of impatient household             
 m_i       =  0.9;                   % LVR for housing borrowing
 vi        =  0.80;                   % interest rate smoothing

vals_etap=1.1;
vals_etay=[-1  0];
vals_etaq =[-1 0 1 ];

 maxcounter=length(vals_etap)*length(vals_etay);
 
 outcounter = 1;
 
maxReps=100;        % number of realizations of shocks
numShocks=7;
nperiods = 300; % number of simulation period

BETA_P1 = cumprod(beta_p*ones(1,nperiods-101));
BETA_P1 = [1 BETA_P1];
BETA_I1 = cumprod(beta_i*ones(1,nperiods-101));
BETA_I1 = [1 BETA_I1];
BETA_P2 = cumprod(beta_p*ones(1,nperiods-401));
BETA_P2 = [1 BETA_P2];
BETA_I2 = cumprod(beta_i*ones(1,nperiods-401));
BETA_I2 = [1 BETA_I2];

U_matrix1 = zeros(3,length(vals_etap),length(vals_etay));
U_matrix2 = zeros(3,length(vals_etap),length(vals_etay));
 
  for num_etap=1:length(vals_etap)
    for num_etay=1:length(vals_etay)
        
          sprintf('Progress: %d/%d',outcounter,maxcounter) 
        
%           Accum_Cp_p = zeros(nperiods,1);
%           Accum_Cp_u = zeros(nperiods,1);
%           Accum_Ci_p = zeros(nperiods,1);
%           Accum_Ci_u = zeros(nperiods,1);
%           
%           Accum_Hp_p = zeros(nperiods,1);
%           Accum_Hp_u = zeros(nperiods,1);
%           Accum_Hi_p = zeros(nperiods,1);
%           Accum_Hi_u = zeros(nperiods,1);
%           
%           Accum_Np_p = zeros(nperiods,1);
%           Accum_Np_u = zeros(nperiods,1);
%           Accum_Ni_p = zeros(nperiods,1);
%           Accum_Ni_u = zeros(nperiods,1);
          
       %[s1] = RandStream.create('mlfg6331_64','NumStreams',maxReps);
        
      for counter = 1:maxReps   
                 
        sprintf('Progress: %d/%d',counter,maxReps) 
 
        eta_p     = vals_etap(num_etap);            % inflation coefficient in the Taylor rule
        eta_y     =   vals_etay(num_etay);          % output growth coefficient in the Taylor rule
        save parameterfile m_i vi eta_p eta_y;  
 
        irfshock =char('dm_i','ehd');%'em','ec','eah','eak','ez','eb'
        
            shockssequence = zeros(nperiods,2);
            shockssequence(1:400,1) = -0.1;%working eb
            %shockssequence(1:100,2) = 0.07;%working eb

        % models and constraints
          modnam      =  'lvrmpnew1newss';         % binding LVR
          modnamstar  =  'lvrmpnew0';     % non-binding LVR
          constraint  =  'GAMAibc < -gammaibcss';  % when yes, switch to NOT binding lvrmp_nb GAMAibc: absolute deviation from gammaibcss
          constraint_relax ='L> LBound'; % when yes, switch to binding lvrmp
        
        % Solve model, generate model IRFs
          [zdatalinear zdatapiecewise zdatass oobase_ Mbase_  ] = ...
          solve_one_constraint(modnam,modnamstar,...
          constraint, constraint_relax,...
          shockssequence,irfshock,nperiods);

        % unpack the data                          
          for i=1:M_.endo_nbr
            eval([deblank(M_.endo_names(i,:)),'_p=zdatapiecewise(:,i);']);
          end
                             
          savefile = sprintf('OUTPUT/SIMUDATA_etay=%g_etap=%g.mat',vals_etay(num_etay),vals_etap(num_etap));
          save(savefile);
          
      end

%         filename = sprintf('SIMUDATA_etay=%g_etap=%g.mat',vals_etay(num_etay),vals_etap(num_etap));
%         
%         load(eval(['filename']));
       
   % calculate life-time utility of tightening LVR
      
          UC_Pp1 = (1/(1-beta_p))*log(cpss) + BETA_P1*Cp_p(1:400); 
          UC_Ip1 = (1/(1-beta_i))*log(ciss) + BETA_I1*Ci_p(1:400); 
          
          UH_Pp1 = (1/(1-beta_p))*log(hpss) + BETA_P1*Hp_p(1:400); 
          UH_Ip1 = (1/(1-beta_i))*log(hiss) + BETA_I1*Hi_p(1:400); 
          
          UN_Pp1 = (1/(1+sigma_n))*BETA_P1*exp(Np_p(1:400));
          UN_Ip1 = (1/(1+sigma_n))*BETA_I1*exp(Ni_p(1:400));
 
          U_Pp1 = UC_Pp1 + xi_hp*UH_Pp1 - xi_np*UN_Pp1 ;
          U_Ip1 = UC_Ip1 + xi_hi*UH_Ip1 - xi_ni*UN_Ip1 ;

          U_matrix1(1,num_etap,num_etay)=U_Pp1;
          U_matrix1(2,num_etap,num_etay)=U_Ip1;
          U_matrix1(3,num_etap,num_etay)=(1-beta_i)*U_Ip1 + (1-beta_p)*U_Pp1;
          
          
    % calculate life-time utility of loosing LVR
      

          UC_Pp2 = (1/(1-beta_p))*log(cpss) + BETA_P2*Cp_p(401:500); 
          UC_Ip2 = (1/(1-beta_i))*log(ciss) + BETA_I2*Ci_p(401:500); 
          
          UH_Pp2 = (1/(1-beta_p))*log(hpss) + BETA_P2*Hp_p(401:500); 
          UH_Ip2 = (1/(1-beta_i))*log(hiss) + BETA_I2*Hi_p(401:500); 
          
          UN_Pp2 = (1/(1+sigma_n))*BETA_P2*exp(Np_p(401:500));
          UN_Ip2 = (1/(1+sigma_n))*BETA_I2*exp(Ni_p(401:500));
 
          U_Pp2 = UC_Pp2 + xi_hp*UH_Pp2 - xi_np*UN_Pp2;
          U_Ip2 = UC_Ip2 + xi_hi*UH_Ip2 - xi_ni*UN_Ip2;

          U_matrix2(1,num_etap,num_etay)=U_Pp2;
          U_matrix2(2,num_etap,num_etay)=U_Ip2;
          U_matrix2(3,num_etap,num_etay)=(1-beta_i)*U_Ip2 + (1-beta_p)*U_Pp2;
                    
         outcounter =  outcounter +1;
  end        
end
        
          savefile = sprintf('OUTPUT/WELFARE1.mat');
          save(savefile, 'U_matrix1', 'U_matrix2','vals_etap','vals_etay'); 
%           
% %plot
% 
surface_u = squeeze(U_matrix1(3,:,:));

surf(vals_etay,vals_etap,surface_u);shading interp     
      
% surface_tightp = squeeze(U_matrixtight(1,:,:));
% surface_tightimp = squeeze(U_matrixtight(2,:,:));
% surface_tighttotal = squeeze(U_matrixtight(3,:,:));
% 
% surface_loosep = squeeze(U_matrixloose(1,:,:));
% surface_looseimp = squeeze(U_matrixloose(2,:,:));
% surface_loosetotal = squeeze(U_matrixloose(3,:,:));
% 
%  [valmax, etapmax] =max(surface_tighttotal)
%  [valmaxmax,etaymax ] =max(valmax)
%  surface_tightimp(etapmax(etaymax),etaymax)
%  surface_tightp(etapmax(etaymax),etaymax)
%  vals_etap(etapmax(etaymax))
%  vals_etay(etaymax)
% 
% [valmax, etapmax] =max(surface_loosetotal)
% [valmaxmax, etaymax] =max(valmax)
% surface_looseimp(etapmax(etaymax),etaymax)
% surface_loosep(etapmax(etaymax),etaymax)
%  vals_etap(etapmax(etaymax))
%  vals_etay(etaymax)



